﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAccordionBus.MovementStrategy;

public abstract class AbstractStrategy
{
    /// <summary>
    /// Перемещаемый объект
    /// </summary>
    private IMoveableObject? _moveableObject;

    /// <summary>
    /// Статус перемещения
    /// </summary>
    private StrategyStatus _state = StrategyStatus.NotInit;

    /// <summary>
    /// Ширина поля
    /// </summary>
    protected int FieldWidth { get; private set; }

    /// <summary>
    /// Высота поля
    /// </summary>
    protected int FieldHeight { get; private set; }

    /// <summary>
    /// Статус перемещения
    /// </summary>
    /// <returns></returns>
    public StrategyStatus GetStatus() { return _state; }

    /// <summary>
    /// Установка данных
    /// </summary>
    /// <param name="moveableObject"></param>
    /// <param name="width"></param>
    /// <param name="height"></param>
    public void SetData(IMoveableObject moveableObject, int width, int height)
    {
        if (moveableObject == null)
        {
            _state = StrategyStatus.NotInit;
            return;
        }
        _state = StrategyStatus.InProgress;
        _moveableObject = moveableObject;
        FieldWidth = width;
        FieldHeight = height;
    }

    public void MakeStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return;
        }

        if (IsTargetDestination())
        {
            _state = StrategyStatus.Finish;
            return;
        }

        MoveToTarget();
    }

    /// <summary>
    /// Перемещение влево
    /// </summary>
    /// <returns>Результат перемещения (true - удалось переместится, false - неудача)</returns>
    protected bool MoveLeft() => MoveTo(MovementDirection.Left);

    /// <summary>
    /// Перемещение вправо
    /// </summary>
    /// <returns>Результат перемещения (true - удалось переместится, false - неудача)</returns>
    protected bool MoveRight() => MoveTo(MovementDirection.Right);

    /// <summary>
    /// Перемещение вверх
    /// </summary>
    /// <returns>Результат перемещения (true - удалось переместится, false - неудача)</returns>
    protected bool MoveUp() => MoveTo(MovementDirection.Up);

    /// <summary>
    /// Перемещение вниз
    /// </summary>
    /// <returns>Результат перемещения (true - удалось переместится, false - неудача)</returns>
    protected bool MoveDown() => MoveTo(MovementDirection.Down);

    /// <summary>
    /// Параметры объекта
    /// </summary>
    protected ObjectParameters? GetObjectParameters => _moveableObject?.GetObjectPosition;

    protected int? GetStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return null;
        }
        return _moveableObject?.GetStep;
    }

    /// <summary>
    /// Перемещение к цели
    /// </summary>
    protected abstract void MoveToTarget();

    protected abstract bool IsTargetDestination();

    protected bool MoveTo(MovementDirection movementDirection)
    {
        if (_state != StrategyStatus.InProgress)
        {
            return false;
        }

        return _moveableObject?.TryMoveObject(movementDirection) ?? false;
    }
}
